use std::collections::HashMap;

#[derive(Debug, Clone, PartialEq)]
pub enum SymbolScope {
    Global,
    Local,
}

#[derive(Debug, Clone)]
pub struct Symbol {
    pub name: String,
    pub scope: SymbolScope,
    pub index: usize,
}

#[derive(Debug, Clone)]
pub struct SymbolTable {
    store: HashMap<String, Symbol>,
    pub count: usize,
    outer: Option<Box<SymbolTable>>,
}

impl SymbolTable {

    pub fn new() -> Self {
        Self {
            store: HashMap::default(),
            count: 0,
            outer: None,
        }
    }

    pub fn outer(&self) -> SymbolTable
    {
        return *self.outer.clone().unwrap()
    }

    pub fn from_outer(outer: SymbolTable) -> Self {
        Self {
            store: HashMap::default(),
            count: 0,
            outer: Some(Box::new(outer)),
        }
    }

    pub fn define(&mut self, name: String) -> &Symbol {
        let symbol = Symbol {
            name: name.clone(),
            index: self.count,
            scope: if self.outer.is_some() { SymbolScope::Local } else { SymbolScope::Global }
        };

        self.store.insert(name.clone(), symbol);
        self.count += 1;

        self.store.get(&name).unwrap()
    }

    pub fn resolve(&self, name: &String) -> Option<&Symbol> {
        let local = self.store.get(name);

        if local.is_none() && self.outer.is_some() {
            self.outer.as_ref().unwrap().resolve(name)
        } else {
            local
        }
    }
}